"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.flushFaults = exports.faults = exports.FAULT_EVENT_TYPE = void 0;

var _highland = _interopRequireDefault(require("highland"));

var uuid = _interopRequireWildcard(require("uuid"));

var _utils = require("../utils");

const FAULT_EVENT_TYPE = 'fault'; // collect faults until the end so that an unhandled error
// does not cause handler faults to be repeatedly published

exports.FAULT_EVENT_TYPE = FAULT_EVENT_TYPE;
const theFaults = [];

const faults = (err, push) => {
  if (err.uow) {
    // handled exceptions are adorned with the uow in error
    // push a fault event onto the stack for publishing by publishFaultsPipeline
    theFaults.push({
      id: uuid.v1(),
      partitionKey: uuid.v4(),
      type: FAULT_EVENT_TYPE,
      timestamp: (0, _utils.now)(),
      tags: {
        functionname: process.env.AWS_LAMBDA_FUNCTION_NAME || 'undefined',
        pipeline: err.uow.pipeline || 'undefined'
      },
      err: {
        name: err.name,
        message: err.message,
        stack: err.stack
      },
      uow: err.uow
    });
    logErr(err);
  } else {
    // rethrow unhandled/unexpected exceptions to stop processing
    push(err);
  }
};

exports.faults = faults;

const flushFaults = opt => s => {
  // use at the every end with through() to redirect to faults streams
  const faultStream = () => {
    const s2 = (0, _highland.default)((push, next) => {
      const f = theFaults.shift();

      if (f) {
        push(null, f);
        next();
      } else {
        push(null, _highland.default.nil);
      }
    });
    return s2 // batch and publish fault events
    .map(fault => ({
      event: fault
    })) // map to uow format
    .through(opt.publish({ ...opt,
      ...opt.faultOpt,
      // override options specific for faults
      handleErrors: false,
      // don't publish faults for faults
      batchSize: Number(process.env.FAULTS_BATCH_SIZE) || Number(process.env.BATCH_SIZE) || 4,
      parallel: Number(process.env.FAULTS_PARALLEL) || Number(process.env.PARALLEL) || 4
    }));
  };

  return s.consume((err, x, push, next) => {
    /* istanbul ignore if */
    if (err) {
      push(err);
      next();
    } else if (x === _highland.default.nil) {
      // this is the purpose of this consume step
      // publish all acuumulated faults at the very end
      next(faultStream());
    } else {
      push(null, x);
      next();
    }
  });
};

exports.flushFaults = flushFaults;

const logErr = err => {
  /* istanbul ignore if */
  if (process.env.AWS_LAMBDA_LOG_GROUP_NAME) {
    if (err instanceof Error) {
      console.error(JSON.stringify({
        errorMessage: err.message,
        errorType: err.name,
        pipeline: err.uow.pipeline || 'undefined',
        handled: err.uow !== undefined,
        stackTrace: err.stack
      }));
    } else {
      console.error(err);
    }
  }
};